@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/functions.h,v 1.34 2009/04/27 18:56:39 amb Exp $
+ $Header: /home/amb/routino/src/RCS/functions.h,v 1.43 2009/10/04 15:53:31 amb Exp $
 
  Header file for function prototypes
 
@@ -37,21 +37,24 @@
 char *FileName(const char *dirname,const char *prefix, const char *name);
 
 void *MapFile(const char *filename);
+void *UnmapFile(const char *filename);
 
 int OpenFile(const char *filename);
-int WriteFile(int fd,void *address,size_t length);
+int ReOpenFile(const char *filename);
+
+int WriteFile(int fd,const void *address,size_t length);
+int ReadFile(int fd,void *address,size_t length);
+int SeekFile(int fd,size_t position);
+
 void CloseFile(int fd);
 
-
-/* In osmparser.c */
-
-int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,Profile *profile);
+int DeleteFile(char *filename);
 
 
 /* In optimiser.c */
 
-Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,index_t finish,Profile *profile,int all);
-Results *FindRoute3(Nodes *supernodes,Segments *supersegments,Ways *superways,Results *begin,Results *end,Profile *profile);
+Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,index_t finish,Profile *profile);
+Results *FindMiddleRoute(Nodes *supernodes,Segments *supersegments,Ways *superways,Results *begin,Results *end,Profile *profile);
 
 Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t start,Profile *profile);
 Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t finish,Profile *profile);
@@ -66,4 +69,11 @@ void PrintRoute(Results *results,Nodes *nodes,Segments *segments,Ways *ways,Prof
 void PrintRouteTail(void);
 
 
+/* In sorting.c */
+
+void filesort(int fd_in,int fd_out,size_t itemsize,size_t ramsize,int (*compare)(const void*,const void*),
+                                                                  int (*buildindex)(void*,index_t));
+void heapsort(void **datap,size_t nitems,int(*compare)(const void*, const void*));
+
+
 #endif /* FUNCTIONS_H */
