@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/nodes.h,v 1.24 2009/05/29 17:45:24 amb Exp $
+ $Header: /home/amb/routino/src/RCS/nodes.h,v 1.29 2009/07/19 14:10:26 amb Exp $
 
  A header file for the nodes.
 
@@ -47,15 +47,16 @@ struct _Node
 /*+ A structure containing a set of nodes (mmap format). +*/
 struct _Nodes
 {
- uint32_t number;               /*+ How many entries are used in total? +*/
+ uint32_t number;               /*+ How many nodes in total? +*/
+ uint32_t snumber;              /*+ How many super-nodes? +*/
 
  uint32_t latbins;              /*+ The number of bins containing latitude. +*/
  uint32_t lonbins;              /*+ The number of bins containing longitude. +*/
 
- int32_t  latzero;              /*+ The latitude of the SW corner of the first bin. +*/
- int32_t  lonzero;              /*+ The longitude of the SW corner of the first bin. +*/
+ ll_bin_t latzero;              /*+ The bin number of the furthest south bin. +*/
+ ll_bin_t lonzero;              /*+ The bin number of the furthest west bin. +*/
 
- index_t *offsets;              /*+ The offset of the first node in each bin. +*/
+ index_t *offsets;              /*+ An array of offset to the first node in each bin. +*/
 
  Node    *nodes;                /*+ An array of nodes. +*/
 
@@ -65,12 +66,14 @@ struct _Nodes
 
 /* Macros */
 
+/*+ Return a Node pointer given a set of nodes and an index. +*/
+#define LookupNode(xxx,yyy)        (&(xxx)->nodes[yyy])
 
 /*+ Return a Segment points given a Node pointer and a set of segments. +*/
 #define FirstSegment(xxx,yyy,zzz)  LookupSegment((xxx),SEGMENT((yyy)->nodes[zzz].firstseg))
 
 /*+ Return true if this is a super-node. +*/
-#define IsSuperNode(xxx,yyy)       (((xxx)->nodes[yyy].firstseg)&SUPER_FLAG)
+#define IsSuperNode(xxx,yyy)       (((xxx)->nodes[yyy].firstseg)&NODE_SUPER)
 
 
 /* Functions */
@@ -78,9 +81,9 @@ struct _Nodes
 
 Nodes *LoadNodeList(const char *filename);
 
-index_t FindNode(Nodes* nodes,Segments *segments,Ways *ways,float latitude,float longitude,distance_t *distance,Profile *profile);
+index_t FindNode(Nodes* nodes,Segments *segments,Ways *ways,double latitude,double longitude,distance_t *distance,Profile *profile);
 
-void GetLatLong(Nodes *nodes,index_t index,float *latitude,float *longitude);
+void GetLatLong(Nodes *nodes,index_t index,double *latitude,double *longitude);
 
 
 #endif /* NODES_H */
