@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/optimiser.c,v 1.70 2009/05/14 18:02:29 amb Exp $
+ $Header: /home/amb/routino/src/RCS/optimiser.c,v 1.76 2009/08/15 15:27:47 amb Exp $
 
  Routing optimiser.
 
@@ -40,9 +40,9 @@ extern int option_quickest;
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Find the optimum route between two nodes.
+  Find the optimum route between two nodes not passing through a super-node.
 
-  Results *FindRoute Returns a set of results.
+  Results *FindNormalRoute Returns a set of results.
 
   Nodes *nodes The set of nodes to use.
 
@@ -55,16 +55,15 @@ extern int option_quickest;
   index_t finish The finish node.
 
   Profile *profile The profile containing the transport type, speeds and allowed highways.
-
-  int all A flag to indicate that a big results structure is required.
   ++++++++++++++++++++++++++++++++++++++*/
 
-Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,index_t finish,Profile *profile,int all)
+Results *FindNormalRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,index_t finish,Profile *profile)
 {
  Results *results;
+ Queue *queue;
  index_t node1,node2;
  score_t finish_score;
- float finish_lat,finish_lon;
+ double  finish_lat,finish_lon;
  Result *result1,*result2;
  Segment *segment;
  Way *way;
@@ -77,10 +76,7 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
 
  /* Create the list of results and insert the first node into the queue */
 
- if(all)
-    results=NewResultsList(65536);
- else
-    results=NewResultsList(8);
+ results=NewResultsList(8);
 
  results->start=start;
  results->finish=finish;
@@ -89,13 +85,15 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
 
  ZeroResult(result1);
 
- insert_in_queue(result1);
+ queue=NewQueueList();
+
+ InsertInQueue(queue,result1);
 
  /* Loop across all nodes in the queue */
 
- while((result1=pop_from_queue()))
+ while((result1=PopFromQueue(queue)))
    {
-    if(result1->sortby>finish_score)
+    if(result1->score>finish_score)
        continue;
 
     node1=result1->node;
@@ -117,6 +115,9 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
        if(result1->prev==node2)
           goto endloop;
 
+       if(node2!=finish && IsSuperNode(nodes,node2))
+          goto endloop;
+
        way=LookupWay(ways,segment->way);
 
        if(!(way->allow&profile->allow))
@@ -125,10 +126,12 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
        if(!profile->highway[HIGHWAY(way->type)])
           goto endloop;
 
-       if(way->weight<profile->weight)
+       if(way->weight && way->weight<profile->weight)
           goto endloop;
 
-       if(way->height<profile->height || way->width<profile->width || way->length<profile->length)
+       if((way->height && way->height<profile->height) ||
+          (way->width  && way->width <profile->width ) ||
+          (way->length && way->length<profile->length))
           goto endloop;
 
        if(option_quickest==0)
@@ -157,18 +160,9 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
             }
           else
             {
-             float lat,lon;
-             distance_t direct;
+             result2->sortby=result2->score;
 
-             GetLatLong(nodes,node2,&lat,&lon);
-             direct=Distance(lat,lon,finish_lat,finish_lon);
-
-             if(option_quickest==0)
-                result2->sortby=result2->score+(score_t)direct/profile->max_pref;
-             else
-                result2->sortby=result2->score+(score_t)distance_speed_to_duration(direct,profile->max_speed)/profile->max_pref;
-
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
        else if(cumulative_score<result2->score) /* New end node is better */
@@ -181,44 +175,22 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
             {
              finish_score=cumulative_score;
             }
-          else if(!all)
-            {
-             insert_in_queue(result2);
-            }
           else
             {
-             float lat,lon;
-             distance_t direct;
+             result2->sortby=result2->score;
 
-             GetLatLong(nodes,node2,&lat,&lon);
-             direct=Distance(lat,lon,finish_lat,finish_lon);
-
-             if(option_quickest==0)
-                result2->sortby=result2->score+(score_t)direct/profile->max_pref;
-             else
-                result2->sortby=result2->score+(score_t)distance_speed_to_duration(direct,profile->max_speed)/profile->max_pref;
-
-             insert_in_queue(result2);
+             if(result2->score<finish_score)
+                InsertInQueue(queue,result2);
             }
          }
 
       endloop:
 
-       if(!option_quiet && !(results->number%10000))
-         {
-          printf("\rRouting: End Nodes=%d",results->number);
-          fflush(stdout);
-         }
-
        segment=NextSegment(segments,segment,node1);
       }
    }
 
- if(!option_quiet)
-   {
-    printf("\rRouted: End Nodes=%d\n",results->number);
-    fflush(stdout);
-   }
+ FreeQueueList(queue);
 
  /* Check it worked */
 
@@ -256,9 +228,9 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Find the optimum route between two nodes.
+  Find the optimum route between two nodes where the start and end are a set of pre-routed super-nodes.
 
-  Results *FindRoute3 Returns a set of results.
+  Results *FindMiddleRoute Returns a set of results.
 
   Nodes *nodes The set of nodes to use.
 
@@ -273,16 +245,23 @@ Results *FindRoute(Nodes *nodes,Segments *segments,Ways *ways,index_t start,inde
   Profile *profile The profile containing the transport type, speeds and allowed highways.
   ++++++++++++++++++++++++++++++++++++++*/
 
-Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Results *end,Profile *profile)
+Results *FindMiddleRoute(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Results *end,Profile *profile)
 {
  Results *results;
+ Queue *queue;
  index_t node1,node2;
  score_t finish_score;
- float finish_lat,finish_lon;
+ double  finish_lat,finish_lon;
  Result *result1,*result2,*result3;
  Segment *segment;
  Way *way;
 
+ if(!option_quiet)
+   {
+    printf("Routing: End Nodes=0");
+    fflush(stdout);
+   }
+
  /* Set up the finish conditions */
 
  finish_score=~(distance_t)0;
@@ -301,36 +280,38 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
 
  *result1=*result3;
 
+ queue=NewQueueList();
+
  /* Insert the finish points of the beginning part of the path into the queue */
 
  result3=FirstResult(begin);
 
  while(result3)
    {
-    if(IsSuperNode(nodes,result3->node))
+    if(result3->node!=begin->start && IsSuperNode(nodes,result3->node))
       {
-       if(!(result2=FindResult(results,result3->node)))
-         {
-          result2=InsertResult(results,result3->node);
+       result2=InsertResult(results,result3->node);
 
-          *result2=*result3;
+       *result2=*result3;
 
-          result2->prev=begin->start;
+       result2->prev=begin->start;
 
-          result2->sortby=result2->score;
-         }
+       result2->sortby=result2->score;
 
-       insert_in_queue(result2);
+       InsertInQueue(queue,result2);
       }
 
     result3=NextResult(begin,result3);
    }
 
+ if(begin->number==1)
+    InsertInQueue(queue,result1);
+
  /* Loop across all nodes in the queue */
 
- while((result1=pop_from_queue()))
+ while((result1=PopFromQueue(queue)))
    {
-    if(result1->sortby>finish_score)
+    if(result1->score>finish_score)
        continue;
 
     node1=result1->node;
@@ -360,10 +341,12 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
        if(!profile->highway[HIGHWAY(way->type)])
           goto endloop;
 
-       if(way->weight<profile->weight)
+       if(way->weight && way->weight<profile->weight)
           goto endloop;
 
-       if(way->height<profile->height || way->width<profile->width || way->length<profile->length)
+       if((way->height && way->height<profile->height) ||
+          (way->width  && way->width <profile->width ) ||
+          (way->length && way->length<profile->length))
           goto endloop;
 
        if(option_quickest==0)
@@ -392,7 +375,7 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
             }
           else
             {
-             float lat,lon;
+             double lat,lon;
              distance_t direct;
 
              GetLatLong(nodes,node2,&lat,&lon);
@@ -403,7 +386,7 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
              else
                 result2->sortby=result2->score+(score_t)distance_speed_to_duration(direct,profile->max_speed)/profile->max_pref;
 
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
        else if(cumulative_score<result2->score) /* New end node is better */
@@ -419,9 +402,9 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
                 finish_score=result2->score+result3->score;
                }
             }
-          else
+          else if(result2->score<finish_score)
             {
-             float lat,lon;
+             double lat,lon;
              distance_t direct;
 
              GetLatLong(nodes,node2,&lat,&lon);
@@ -432,7 +415,7 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
              else
                 result2->sortby=result2->score+(score_t)distance_speed_to_duration(direct,profile->max_speed)/profile->max_pref;
 
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
 
@@ -481,6 +464,8 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
       }
    }
 
+ FreeQueueList(queue);
+
  /* Check it worked */
 
  result2=FindResult(results,end->finish);
@@ -533,6 +518,7 @@ Results *FindRoute3(Nodes *nodes,Segments *segments,Ways *ways,Results *begin,Re
 Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t start,Profile *profile)
 {
  Results *results;
+ Queue *queue;
  index_t node1,node2;
  Result *result1,*result2;
  Segment *segment;
@@ -548,11 +534,13 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 
  ZeroResult(result1);
 
- insert_in_queue(result1);
+ queue=NewQueueList();
+
+ InsertInQueue(queue,result1);
 
  /* Loop across all nodes in the queue */
 
- while((result1=pop_from_queue()))
+ while((result1=PopFromQueue(queue)))
    {
     node1=result1->node;
 
@@ -581,10 +569,12 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
        if(!profile->highway[HIGHWAY(way->type)])
           goto endloop;
 
-       if(way->weight<profile->weight)
+       if(way->weight && way->weight<profile->weight)
           goto endloop;
 
-       if(way->height<profile->height || way->width<profile->width || way->length<profile->length)
+       if((way->height && way->height<profile->height) ||
+          (way->width  && way->width <profile->width ) ||
+          (way->length && way->length<profile->length))
           goto endloop;
 
        if(option_quickest==0)
@@ -607,7 +597,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
           if(!IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
        else if(cumulative_score<result2->score) /* New end node is better */
@@ -619,7 +609,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
           if(!IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
 
@@ -629,6 +619,8 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
       }
    }
 
+ FreeQueueList(queue);
+
  /* Check it worked */
 
  if(results->number==1)
@@ -660,6 +652,7 @@ Results *FindStartRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t star
 Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t finish,Profile *profile)
 {
  Results *results;
+ Queue *queue;
  index_t node1,node2;
  Result *result1,*result2;
  Segment *segment;
@@ -675,11 +668,13 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
 
  ZeroResult(result1);
 
- insert_in_queue(result1);
+ queue=NewQueueList();
+
+ InsertInQueue(queue,result1);
 
  /* Loop across all nodes in the queue */
 
- while((result1=pop_from_queue()))
+ while((result1=PopFromQueue(queue)))
    {
     node1=result1->node;
 
@@ -708,10 +703,12 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
        if(!profile->highway[HIGHWAY(way->type)])
           goto endloop;
 
-       if(way->weight<profile->weight)
+       if(way->weight && way->weight<profile->weight)
           goto endloop;
 
-       if(way->height<profile->height || way->width<profile->width || way->length<profile->length)
+       if((way->height && way->height<profile->height) ||
+          (way->width  && way->width <profile->width ) ||
+          (way->length && way->length<profile->length))
           goto endloop;
 
        if(option_quickest==0)
@@ -734,7 +731,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
           if(!IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
        else if(cumulative_score<result2->score) /* New end node is better */
@@ -746,7 +743,7 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
           if(!IsSuperNode(nodes,node2))
             {
              result2->sortby=result2->score;
-             insert_in_queue(result2);
+             InsertInQueue(queue,result2);
             }
          }
 
@@ -756,6 +753,8 @@ Results *FindFinishRoutes(Nodes *nodes,Segments *segments,Ways *ways,index_t fin
       }
    }
 
+ FreeQueueList(queue);
+
  /* Check it worked */
 
  if(results->number==1)
@@ -788,17 +787,12 @@ Results *CombineRoutes(Results *results,Nodes *nodes,Segments *segments,Ways *wa
 {
  Result *result1,*result2,*result3,*result4;
  Results *combined;
- int quiet=option_quiet;
 
  combined=NewResultsList(64);
 
  combined->start=results->start;
  combined->finish=results->finish;
 
- /* Don't print any output for this part */
-
- option_quiet=1;
-
  /* Sort out the combined route */
 
  result1=FindResult(results,results->start);
@@ -811,7 +805,7 @@ Results *CombineRoutes(Results *results,Nodes *nodes,Segments *segments,Ways *wa
    {
     if(result1->next!=NO_NODE)
       {
-       Results *results2=FindRoute(nodes,segments,ways,result1->node,result1->next,profile,0);
+       Results *results2=FindNormalRoute(nodes,segments,ways,result1->node,result1->next,profile);
 
        result2=FindResult(results2,result1->node);
 
@@ -844,9 +838,5 @@ Results *CombineRoutes(Results *results,Nodes *nodes,Segments *segments,Ways *wa
    }
  while(result1);
 
- /* Now print out the result normally */
-
- option_quiet=quiet;
-
  return(combined);
 }
