@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/osmparser.c,v 1.37 2009/06/05 16:39:11 amb Exp $
+ $Header: /home/amb/routino/src/RCS/osmparser.c,v 1.51 2009/10/10 16:21:19 amb Exp $
 
  OSM XML file parser (either JOSM or planet)
 
@@ -26,16 +26,19 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <math.h>
 
-#include "types.h"
-#include "functions.h"
+#include "typesx.h"
+#include "functionsx.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
+#include "ways.h"
 
 
-#define BUFFSIZE 64
+/*+ The length of the buffer and the size increment for reading lines from the file. +*/
+#define BUFFSIZE 256
+
+/* Local functions */
 
 static char *fgets_realloc(char *buffer,FILE *file);
 
@@ -53,7 +56,7 @@ static char *fgets_realloc(char *buffer,FILE *file);
 
   WaysX *OSMWays The arrray of ways to fill in.
 
-  Profile profile A profile of the allowed transport types and included/excluded highway types.
+  Profile *profile A profile of the allowed transport types and included/excluded highway types.
   ++++++++++++++++++++++++++++++++++++++*/
 
 int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,Profile *profile)
@@ -62,6 +65,7 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
  long nlines=0;
  long nnodes=0,nways=0,nrelations=0;
  int isnode=0,isway=0,isrelation=0;
+ way_t way_id=0;
  int way_oneway=0,way_roundabout=0;
  speed_t way_maxspeed=0;
  weight_t way_maxweight=0;
@@ -73,6 +77,9 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
  node_t *way_nodes=NULL;
  int way_nnodes=0,way_nalloc=0;
 
+ printf("\rReading: Lines=0 Nodes=0 Ways=0 Relations=0");
+ fflush(stdout);
+
  /* Parse the file */
 
  while((line=fgets_realloc(line,file)))
@@ -87,15 +94,15 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
     if(!strncmp(l,"<node",5)) /* The start of a node */
       {
        node_t id;
-       float latitude,longitude;
+       double latitude,longitude;
 
        nnodes++;
 
        isnode=1; isway=0; isrelation=0;
 
        m=strstr(l,"id=");  m+=4; if(*m=='"' || *m=='\'') m++; id=atoll(m);
-       m=strstr(l,"lat="); m+=5; if(*m=='"' || *m=='\'') m++; latitude=(M_PI/180)*atof(m);
-       m=strstr(l,"lon="); m+=4; if(*m=='"' || *m=='\'') m++; longitude=(M_PI/180)*atof(m);
+       m=strstr(l,"lat="); m+=5; if(*m=='"' || *m=='\'') m++; latitude=degrees_to_radians(atof(m));
+       m=strstr(l,"lon="); m+=4; if(*m=='"' || *m=='\'') m++; longitude=degrees_to_radians(atof(m));
 
        AppendNode(OSMNodes,id,latitude,longitude);
 
@@ -114,6 +121,8 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
 
        isnode=0; isway=1; isrelation=0;
 
+       m=strstr(l,"id=");  m+=4; if(*m=='"' || *m=='\'') m++; way_id=atoll(m);
+
        way_oneway=0; way_roundabout=0;
        way_maxspeed=0; way_maxweight=0; way_maxheight=0; way_maxwidth=0;
        way_maxlength=0;
@@ -186,7 +195,7 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
 
           if(allow&profile->allow && profile->highway[HIGHWAY(type)])
             {
-             Way *way;
+             Way way={0};
              char *refname;
              int i;
 
@@ -212,23 +221,23 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
              else /* if(!way_ref && !way_name && !way_roundabout) */
                 refname=way_highway;
 
-             way=AppendWay(OSMWays,refname);
+             way.speed=way_maxspeed;
+             way.weight=way_maxweight;
+             way.height=way_maxheight;
+             way.width=way_maxwidth;
+             way.length=way_maxlength;
 
-             way->speed=way_maxspeed;
-             way->weight=way_maxweight;
-             way->height=way_maxheight;
-             way->width=way_maxwidth;
-             way->length=way_maxlength;
+             way.type=type;
 
-             way->type=type;
-
-             way->allow=allow;
+             way.allow=allow;
 
              if(way_oneway)
-                way->type|=Way_OneWay;
+                way.type|=Way_OneWay;
 
              if(way_roundabout)
-                way->type|=Way_Roundabout;
+                way.type|=Way_Roundabout;
+
+             AppendWay(OSMWays,way_id,&way,refname);
 
              if(refname!=way_ref && refname!=way_name && refname!=way_highway)
                 free(refname);
@@ -237,23 +246,22 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
                {
                 node_t from=way_nodes[i-1];
                 node_t to  =way_nodes[i];
-                Segment *segment;
-
-                segment=AppendSegment(OSMSegments,from,to);
-                segment->way=OSMWays->number-1;
 
                 if(way_oneway>0)
-                   segment->distance=ONEWAY_1TO2;
+                  {
+                   AppendSegment(OSMSegments,way_id,from,to,ONEWAY_1TO2);
+                   AppendSegment(OSMSegments,way_id,to,from,ONEWAY_2TO1);
+                  }
                 else if(way_oneway<0)
-                   segment->distance=ONEWAY_2TO1;
-
-                segment=AppendSegment(OSMSegments,to,from);
-                segment->way=OSMWays->number-1;
-
-                if(way_oneway>0)
-                   segment->distance=ONEWAY_2TO1;
-                else if(way_oneway<0)
-                   segment->distance=ONEWAY_1TO2;
+                  {
+                   AppendSegment(OSMSegments,way_id,from,to,ONEWAY_2TO1);
+                   AppendSegment(OSMSegments,way_id,to,from,ONEWAY_1TO2);
+                  }
+                else
+                  {
+                   AppendSegment(OSMSegments,way_id,from,to,0);
+                   AppendSegment(OSMSegments,way_id,to,from,0);
+                  }
                }
             }
          }
@@ -283,8 +291,8 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
          {
           m=strstr(l,"ref="); m+=4; if(*m=='"' || *m=='\'') m++; id=atoll(m);
 
-          if(way_nnodes<=way_nalloc)
-             way_nodes=(node_t*)realloc((void*)way_nodes,(way_nalloc+=16)*sizeof(node_t));
+          if(way_nnodes==way_nalloc)
+             way_nodes=(node_t*)realloc((void*)way_nodes,(way_nalloc+=256)*sizeof(node_t));
 
           way_nodes[way_nnodes++]=id;
          }
@@ -394,9 +402,9 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
                    int feet,inches;
 
                    if(sscanf(v,"%d'%d\"",&feet,&inches)==2)
-                      way_maxheight=metres_to_height((feet+(float)inches/12.0)*0.254);
+                      way_maxheight=metres_to_height((feet+(double)inches/12.0)*0.254);
                    else if(sscanf(v,"%d'",&feet)==1)
-                      way_maxheight=metres_to_height((feet+(float)inches/12.0)*0.254);
+                      way_maxheight=metres_to_height((feet+(double)inches/12.0)*0.254);
                   }
                 else if(strstr(v,"ft") || strstr(v,"feet"))
                    way_maxheight=metres_to_height(atof(v)*0.254);
@@ -410,9 +418,9 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
                    int feet,inches;
 
                    if(sscanf(v,"%d'%d\"",&feet,&inches)==2)
-                      way_maxwidth=metres_to_height((feet+(float)inches/12.0)*0.254);
+                      way_maxwidth=metres_to_height((feet+(double)inches/12.0)*0.254);
                    else if(sscanf(v,"%d'",&feet)==1)
-                      way_maxwidth=metres_to_height((feet+(float)inches/12.0)*0.254);
+                      way_maxwidth=metres_to_height((feet+(double)inches/12.0)*0.254);
                   }
                 else if(strstr(v,"ft") || strstr(v,"feet"))
                    way_maxwidth=metres_to_width(atof(v)*0.254);
@@ -426,9 +434,9 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
                    int feet,inches;
 
                    if(sscanf(v,"%d'%d\"",&feet,&inches)==2)
-                      way_maxlength=metres_to_height((feet+(float)inches/12.0)*0.254);
+                      way_maxlength=metres_to_height((feet+(double)inches/12.0)*0.254);
                    else if(sscanf(v,"%d'",&feet)==1)
-                      way_maxlength=metres_to_height((feet+(float)inches/12.0)*0.254);
+                      way_maxlength=metres_to_height((feet+(double)inches/12.0)*0.254);
                   }
                 else if(strstr(v,"ft") || strstr(v,"feet"))
                    way_maxlength=metres_to_length(atof(v)*0.254);
@@ -449,6 +457,13 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
                 else
                    way_allow_no|=Allow_Motorcar;
                }
+             if(!strcmp(k,"motor_vehicle"))
+               {
+                if(!strcmp(v,"true") || !strcmp(v,"yes") || !strcmp(v,"1") || !strcmp(v,"permissive") || !strcmp(v,"designated"))
+                   way_allow_yes|=Allow_Motorbike|Allow_Motorcar|Allow_Goods|Allow_HGV|Allow_PSV;
+                else
+                   way_allow_no|=Allow_Motorbike|Allow_Motorcar|Allow_Goods|Allow_HGV|Allow_PSV;
+               }
              break;
 
             case 'n':
@@ -481,6 +496,16 @@ int ParseXML(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,P
                 way_ref=strcpy((char*)malloc(strlen(v)+1),v);
              break;
 
+            case 'v':
+             if(!strcmp(k,"vehicle"))
+               {
+                if(!strcmp(v,"true") || !strcmp(v,"yes") || !strcmp(v,"1") || !strcmp(v,"permissive") || !strcmp(v,"designated"))
+                   way_allow_yes|=Allow_Bicycle|Allow_Motorbike|Allow_Motorcar|Allow_Goods|Allow_HGV|Allow_PSV;
+                else
+                   way_allow_no|=Allow_Bicycle|Allow_Motorbike|Allow_Motorcar|Allow_Goods|Allow_HGV|Allow_PSV;
+               }
+             break;
+
             default:
              ;
             }
