@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/planetsplitter.c,v 1.37 2009/06/13 13:02:12 amb Exp $
+ $Header: /home/amb/routino/src/RCS/planetsplitter.c,v 1.61 2009/10/12 17:35:26 amb Exp $
 
  OSM planet file splitter.
 
@@ -26,24 +26,36 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "typesx.h"
 #include "types.h"
+#include "functionsx.h"
 #include "functions.h"
 #include "nodesx.h"
 #include "segmentsx.h"
 #include "waysx.h"
 #include "superx.h"
 #include "profiles.h"
+#include "ways.h"
+
+
+/* Variables */
+
+/*+ The option to use a slim mode with file-backed read-only intermediate storage. +*/
+int option_slim=0;
+
+/*+ The name of the temporary directory. +*/
+char *option_tmpdirname=NULL;
 
 
 int main(int argc,char** argv)
 {
- NodesX *OSMNodes;
- SegmentsX *OSMSegments,*SuperSegments=NULL;
- WaysX *OSMWays;
+ NodesX *Nodes;
+ SegmentsX *Segments,*SuperSegments=NULL,*MergedSegments=NULL;
+ WaysX *Ways;
  int iteration=0,quit=0;
  int max_iterations=10;
  char *dirname=NULL,*prefix=NULL;
- Profile profile;
+ Profile profile={0};
  int i;
 
  /* Fill in the default profile. */
@@ -55,9 +67,6 @@ int main(int argc,char** argv)
  for(i=1;i<Way_Unknown;i++)
     profile.highway[i]=1;
 
- for(i=1;i<Way_Unknown;i++)
-    profile.speed[i]=0; /* Not used by planetsplitter */
-
  profile.oneway=1; /* Not used by planetsplitter */
 
  /* Parse the command line arguments */
@@ -66,8 +75,12 @@ int main(int argc,char** argv)
    {
     if(!strcmp(argv[argc],"--help"))
        goto usage;
+    else if(!strcmp(argv[argc],"--slim"))
+       option_slim=1;
     else if(!strncmp(argv[argc],"--dir=",6))
        dirname=&argv[argc][6];
+    else if(!strncmp(argv[argc],"--tmpdir=",9))
+       option_tmpdirname=&argv[argc][9];
     else if(!strncmp(argv[argc],"--prefix=",9))
        prefix=&argv[argc][9];
     else if(!strncmp(argv[argc],"--max-iterations=",17))
@@ -88,6 +101,7 @@ int main(int argc,char** argv)
 
        fprintf(stderr,"Usage: planetsplitter\n"
                       "                      [--help]\n"
+                      "                      [--slim] [--tmpdir=<name>]\n"
                       "                      [--dir=<name>] [--prefix=<name>]\n"
                       "                      [--max-iterations=<number>]\n"
                       "                      [--transport=<transport>]\n"
@@ -104,64 +118,71 @@ int main(int argc,char** argv)
       }
    }
 
- /* Create new variables */
+ if(!option_tmpdirname)
+   {
+    if(!dirname)
+       option_tmpdirname=".";
+    else
+       option_tmpdirname=dirname;
+   }
 
- OSMNodes=NewNodeList();
- OSMSegments=NewSegmentList();
- OSMWays=NewWayList();
+ /* Create new node, segment and way variables */
+
+ Nodes=NewNodeList();
+
+ Segments=NewSegmentList();
+
+ Ways=NewWayList();
 
  /* Parse the file */
 
- printf("\nParsing OSM Data\n================\n\n"); fflush(stdout);
+ printf("\nParse OSM Data\n==============\n\n");
+ fflush(stdout);
 
- ParseXML(stdin,OSMNodes,OSMSegments,OSMWays,&profile);
+ ParseXML(stdin,Nodes,Segments,Ways,&profile);
 
- printf("\nProcessing OSM Data\n===================\n\n"); fflush(stdout);
+ /* Process the data */
 
- /* Sort the ways */
+ printf("\nProcess OSM Data\n================\n\n");
+ fflush(stdout);
 
- SortWayList(OSMWays);
+ /* Sort the nodes, segments and ways */
 
- /* Sort the segments */
+ SortNodeList(Nodes);
 
- SortSegmentList(OSMSegments);
+ SortSegmentList(Segments);
 
- /* Sort the nodes */
+ SortWayList(Ways);
 
- SortNodeList(OSMNodes);
+ /* Remove bad segments (must be after sorting the nodes and segments) */
 
- /* Remove bad segments */
+ RemoveBadSegments(Nodes,Segments);
 
- RemoveBadSegments(OSMNodes,OSMSegments);
+ /* Remove non-highway nodes (must be after removing the bad segments) */
 
- SortSegmentList(OSMSegments);
+ RemoveNonHighwayNodes(Nodes,Segments);
 
- /* Remove non-way nodes */
+ /* Measure the segments and replace node/way id with index (must be after removing non-highway nodes) */
 
- RemoveNonHighwayNodes(OSMNodes,OSMSegments);
-
- SortNodeList(OSMNodes);
-
- /* Measure the segments */
-
- MeasureSegments(OSMSegments,OSMNodes);
+ UpdateSegments(Segments,Nodes,Ways);
 
 
- /* Repeated iteration on Super-Nodes, Super-Segments and Super-Ways */
+ /* Repeated iteration on Super-Nodes and Super-Segments */
 
  do
    {
-    printf("\nProcessing Super-Data (iteration %d)\n===================================%s\n\n",iteration,iteration>10?"=":""); fflush(stdout);
+    printf("\nProcess Super-Data (iteration %d)\n================================%s\n\n",iteration,iteration>9?"=":"");
+    fflush(stdout);
 
     if(iteration==0)
       {
        /* Select the super-nodes */
 
-       ChooseSuperNodes(OSMNodes,OSMSegments,OSMWays,iteration);
+       ChooseSuperNodes(Nodes,Segments,Ways);
 
        /* Select the super-segments */
 
-       SuperSegments=CreateSuperSegments(OSMNodes,OSMSegments,OSMWays,iteration);
+       SuperSegments=CreateSuperSegments(Nodes,Segments,Ways,iteration);
       }
     else
       {
@@ -169,13 +190,13 @@ int main(int argc,char** argv)
 
        /* Select the super-nodes */
 
-       ChooseSuperNodes(OSMNodes,SuperSegments,OSMWays,iteration);
+       ChooseSuperNodes(Nodes,SuperSegments,Ways);
 
        /* Select the super-segments */
 
-       SuperSegments2=CreateSuperSegments(OSMNodes,SuperSegments,OSMWays,iteration);
+       SuperSegments2=CreateSuperSegments(Nodes,SuperSegments,Ways,iteration);
 
-       if(SuperSegments->number==SuperSegments2->number)
+       if(SuperSegments->xnumber==SuperSegments2->xnumber)
           quit=1;
 
        FreeSegmentList(SuperSegments);
@@ -187,6 +208,10 @@ int main(int argc,char** argv)
 
     SortSegmentList(SuperSegments);
 
+    /* Remove duplicated super-segments */
+
+    DeduplicateSegments(SuperSegments,Nodes,Ways);
+
     iteration++;
 
     if(iteration>max_iterations)
@@ -194,55 +219,82 @@ int main(int argc,char** argv)
    }
  while(!quit);
 
- printf("\n"); fflush(stdout);
+ /* Combine the super-segments */
 
- /* Mark the super-nodes */
-
- MarkSuperNodes(OSMNodes,iteration);
+ printf("\nCombine Segments and Super-Segments\n===================================\n\n");
+ fflush(stdout);
 
  /* Merge the super-segments */
 
- MergeSuperSegments(OSMSegments,SuperSegments);
+ MergedSegments=MergeSuperSegments(Segments,SuperSegments);
+
+ FreeSegmentList(Segments);
 
  FreeSegmentList(SuperSegments);
 
- /* Sort the segments */
-
- SortSegmentList(OSMSegments);
+ Segments=MergedSegments;
 
  /* Rotate segments so that node1<node2 */
 
- RotateSegments(OSMSegments,OSMNodes);
+ RotateSegments(Segments);
 
  /* Sort the segments */
 
- SortSegmentList(OSMSegments);
+ SortSegmentList(Segments);
 
  /* Remove duplicated segments */
 
- DeduplicateSegments(OSMSegments,OSMNodes,OSMWays);
+ DeduplicateSegments(Segments,Nodes,Ways);
 
- /* Sort the segments */
+ /* Cross reference the nodes and segments */
 
- SortSegmentList(OSMSegments);
+ printf("\nCross-Reference Nodes and Segments\n==================================\n\n");
+ fflush(stdout);
+
+ /* Compact the ways */
+
+ CompactWayNames(Ways);
+
+ CompactWayProperties(Ways);
+
+ /* Sort the node list geographically */
+
+ SortNodeListGeographically(Nodes);
+
+ /* Create the real segments and nodes */
+
+ CreateRealNodes(Nodes,iteration);
+
+ CreateRealSegments(Segments,Ways);
 
  /* Fix the segment and node indexes */
 
- IndexNodes(OSMNodes,OSMSegments);
+ IndexNodes(Nodes,Segments);
 
- IndexSegments(OSMSegments,OSMNodes);
+ IndexSegments(Segments,Nodes);
+
+ /* Output the results */
+
+ printf("\nWrite Out Database Files\n========================\n\n");
+ fflush(stdout);
 
  /* Write out the nodes */
 
- SaveNodeList(OSMNodes,FileName(dirname,prefix,"nodes.mem"));
+ SaveNodeList(Nodes,FileName(dirname,prefix,"nodes.mem"));
+
+ FreeNodeList(Nodes);
 
  /* Write out the segments */
 
- SaveSegmentList(OSMSegments,FileName(dirname,prefix,"segments.mem"));
+ SaveSegmentList(Segments,FileName(dirname,prefix,"segments.mem"));
+
+ FreeSegmentList(Segments);
 
  /* Write out the ways */
 
- SaveWayList(OSMWays,FileName(dirname,prefix,"ways.mem"));
+ SaveWayList(Ways,FileName(dirname,prefix,"ways.mem"));
+
+ FreeWayList(Ways);
 
  return(0);
 }
