@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/profiles.c,v 1.16 2009/05/23 16:55:14 amb Exp $
+ $Header: /home/amb/routino/src/RCS/profiles.c,v 1.18 2009/07/12 09:01:48 amb Exp $
 
  The pre-defined profiles and the functions for handling them.
 
@@ -29,6 +29,7 @@
 #include "ways.h"
 
 
+/*+ The set of built-in profiles for different transport types. +*/
 static Profile builtin_profiles[]=
  {
   /* The profile for travel by Foot */
@@ -430,7 +431,7 @@ void UpdateProfile(Profile *profile)
 
 void PrintProfile(const Profile *profile)
 {
- int i;
+ unsigned int i;
 
  printf("Profile\n=======\n");
 
@@ -465,7 +466,7 @@ void PrintProfile(const Profile *profile)
 
 void PrintProfilesJS(void)
 {
- int i,j;
+ unsigned int i,j;
 
  printf("// Transport types\n");
  printf("var router_transports={");
@@ -542,7 +543,7 @@ void PrintProfilesJS(void)
 
 void PrintProfilesPerl(void)
 {
- int i,j;
+ unsigned int i,j;
 
  printf("# Transport types\n");
  printf("@router_transports=(");
