@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/results.c,v 1.14 2009/05/14 18:02:30 amb Exp $
+ $Header: /home/amb/routino/src/RCS/results.c,v 1.19 2009/08/15 14:18:23 amb Exp $
 
  Result data type functions.
 
@@ -22,28 +22,14 @@
  ***************************************/
 
 
-#include <assert.h>
+#include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "results.h"
 
-
+/*+ The size of the increment for the Results data structure. +*/
 #define RESULTS_INCREMENT    16
-#define QUEUE_INCREMENT   10240
-
-
-/*+ A queue of results. +*/
-typedef struct _Queue
-{
- uint32_t  alloced;             /*+ The amount of space allocated for results in the array. +*/
- uint32_t  number;              /*+ The number of occupied results in the array. +*/
- Result  **xqueue;              /*+ An array of pointers to parts of the results structure. +*/
-}
- Queue;
-
-/*+ The queue of nodes. +*/
-static Queue queue={0,0,NULL};
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -57,7 +43,7 @@ static Queue queue={0,0,NULL};
 Results *NewResultsList(int nbins)
 {
  Results *results;
- int i;
+ uint32_t i;
 
  results=(Results*)malloc(sizeof(Results));
 
@@ -103,8 +89,7 @@ Results *NewResultsList(int nbins)
 
 void FreeResultsList(Results *results)
 {
- int c=(results->number-1)/(results->nbins*RESULTS_INCREMENT);
- int i;
+ int i,c=(results->number-1)/(results->nbins*RESULTS_INCREMENT);
 
  for(i=c;i>=0;i--)
     free(results->data[i]);
@@ -125,7 +110,7 @@ void FreeResultsList(Results *results)
 /*++++++++++++++++++++++++++++++++++++++
   Insert a new result into the results data structure in the right order.
 
-  Result *insert_result Returns the result that has been inserted.
+  Result *InsertResult Returns the result that has been inserted.
 
   Results *results The results structure to insert into.
 
@@ -135,7 +120,7 @@ void FreeResultsList(Results *results)
 Result *InsertResult(Results *results,index_t node)
 {
  int bin=node&results->mask;
- int i;
+ uint32_t i;
 
  /* Check that the arrays have enough space. */
 
@@ -164,6 +149,7 @@ Result *InsertResult(Results *results,index_t node)
  results->count[bin]++;
 
  results->point[bin][results->count[bin]-1]->node=node;
+ results->point[bin][results->count[bin]-1]->queued=~0;
 
  return(results->point[bin][results->count[bin]-1]);
 }
@@ -190,7 +176,7 @@ void ZeroResult(Result *result)
 /*++++++++++++++++++++++++++++++++++++++
   Find a result; search by node.
 
-  Result *insert_result Returns the result that has been found.
+  Result *FindResult Returns the result that has been found.
 
   Results *results The results structure to search.
 
@@ -236,12 +222,11 @@ Result *FirstResult(Results *results)
 
 Result *NextResult(Results *results,Result *result)
 {
- int c=(results->number-1)/(results->nbins*RESULTS_INCREMENT);
- int i,j=0;
+ int i,j=0,c=(results->number-1)/(results->nbins*RESULTS_INCREMENT);
 
  for(i=0;i<=c;i++)
    {
-    j=(result-results->data[i]);
+    j=result-results->data[i];
 
     if(j>=0 && j<(results->nbins*RESULTS_INCREMENT))
        break;
@@ -255,107 +240,3 @@ Result *NextResult(Results *results,Result *result)
 
  return(&results->data[i][j]);
 }
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Insert an item into the queue in the right order.
-
-  Result *result The result to insert into the queue.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void insert_in_queue(Result *result)
-{
- int start=0;
- int end=queue.number-1;
- int mid;
- int insert=-1;
-
- /* Check that the array is allocated. */
-
- if(!queue.xqueue)
-   {
-    queue.alloced=QUEUE_INCREMENT;
-    queue.number=0;
-    queue.xqueue=(Result**)malloc(queue.alloced*sizeof(Result*));
-   }
-
- /* Check that the arrays have enough space. */
-
- if(queue.number==queue.alloced)
-   {
-    queue.alloced+=QUEUE_INCREMENT;
-    queue.xqueue=(Result**)realloc((void*)queue.xqueue,queue.alloced*sizeof(Result*));
-   }
-
- /* Binary search - search key may not match, new insertion point required
-  *
-  *  # <- start  |  Check mid and move start or end if it doesn't match
-  *  #           |
-  *  #           |  Since there may not be an exact match we must set end=mid
-  *  # <- mid    |  or start=mid because we know that mid doesn't match.
-  *  #           |
-  *  #           |  Eventually end=start+1 and the insertion point is before
-  *  # <- end    |  end (since it cannot be before the initial start or end).
-  */
-
- if(queue.number==0)                                 /* There is nothing in the queue */
-    insert=0;
- else if(result->sortby>queue.xqueue[start]->sortby) /* Check key is not before start */
-    insert=start;
- else if(result->sortby<queue.xqueue[end]->sortby)   /* Check key is not after end */
-    insert=end+1;
- else if(queue.number==2)                            /* Must be between them */
-    insert=1;
- else
-   {
-    do
-      {
-       mid=(start+end)/2;                                /* Choose mid point */
-
-       if(queue.xqueue[mid]->sortby>result->sortby)      /* Mid point is too low */
-          start=mid;
-       else if(queue.xqueue[mid]->sortby<result->sortby) /* Mid point is too high */
-          end=mid;
-       else                                              /* Mid point is correct */
-         {
-          if(queue.xqueue[mid]==result)
-             return;
-
-          insert=mid;
-          break;
-         }
-      }
-    while((end-start)>1);
-
-    if(insert==-1)
-       insert=end;
-   }
-
- /* Shuffle the array up */
-
- if(insert!=queue.number)
-    memmove(&queue.xqueue[insert+1],&queue.xqueue[insert],(queue.number-insert)*sizeof(Result*));
-
- /* Insert the new entry */
-
- queue.xqueue[insert]=result;
-
- queue.number++;
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Pop an item from the end of the queue.
-
-  Result *pop_from_queue Returns the top item.
-
-  Results *results The set of results that the queue is processing.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-Result *pop_from_queue()
-{
- if(queue.number)
-    return(queue.xqueue[--queue.number]);
- else
-    return(NULL);
-}
