@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/results.h,v 1.14 2009/05/13 17:45:31 amb Exp $
+ $Header: /home/amb/routino/src/RCS/results.h,v 1.16 2009/08/15 14:18:23 amb Exp $
 
  A header file for the results.
 
@@ -40,8 +40,10 @@ typedef struct _Result
  index_t   prev;                /*+ The previous node following the best path. +*/
  index_t   next;                /*+ The next node following the best path. +*/
 
- score_t   score;               /*+ The weighted distance or duration score to the node following the best path. +*/
- score_t   sortby;              /*+ The value to sort the results by in the queue. +*/
+ score_t   score;               /*+ The best actual weighted distance or duration score from the start to the node. +*/
+
+ score_t   sortby;              /*+ The best possible weighted distance or duration score from the start to the finish. +*/
+ uint32_t  queued;              /*+ The position of this result in the queue. +*/
 
  Segment  *segment;             /*+ The segment for the path to here (from prev). +*/
 }
@@ -73,7 +75,12 @@ typedef struct _Results
  Results;
 
 
-/* Functions */
+/* Forward definitions for opaque type */
+
+typedef struct _Queue Queue;
+
+
+/* Results Functions */
 
 Results *NewResultsList(int nbins);
 void FreeResultsList(Results *results);
@@ -86,10 +93,14 @@ Result *FindResult(Results *results,index_t node);
 Result *FirstResult(Results *results);
 Result *NextResult(Results *results,Result *result);
 
+
 /* Queue Functions */
 
-void insert_in_queue(Result *result);
-Result *pop_from_queue(void);
+Queue *NewQueueList(void);
+void FreeQueueList(Queue *queue);
+
+void InsertInQueue(Queue *queue,Result *result);
+Result *PopFromQueue(Queue *queue);
 
 
 #endif /* RESULTS_H */
