@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/router.c,v 1.52 2009/05/29 17:45:24 amb Exp $
+ $Header: /home/amb/routino/src/RCS/router.c,v 1.58 2009/08/17 18:26:22 amb Exp $
 
  OSM router.
 
@@ -25,7 +25,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
 #include <ctype.h>
 
 #include "types.h"
@@ -36,6 +35,10 @@
 #include "ways.h"
 
 
+/*+ The number of waypoints allowed to be specified. +*/
+#define NWAYPOINTS 99
+
+
 /*+ The option not to print any progress information. +*/
 int option_quiet=0;
 
@@ -48,11 +51,10 @@ int main(int argc,char** argv)
  Nodes    *OSMNodes;
  Segments *OSMSegments;
  Ways     *OSMWays;
- Results  *results[10]={NULL};
- int       point_used[10]={0};
- float     point_lon[10],point_lat[10];
+ Results  *results[NWAYPOINTS+1]={NULL};
+ int       point_used[NWAYPOINTS+1]={0};
+ double    point_lon[NWAYPOINTS+1],point_lat[NWAYPOINTS+1];
  int       help_profile=0,help_profile_js=0,help_profile_pl=0;
- int       option_all=0,option_super=0,option_no_output=0;
  char     *dirname=NULL,*prefix=NULL,*filename;
  Transport transport=Transport_None;
  Profile   profile;
@@ -67,12 +69,11 @@ int main(int argc,char** argv)
 
     fprintf(stderr,"Usage: router [--lon1=]<longitude> [--lat1=]<latitude>\n"
                    "              [--lon2=]<longitude> [--lon2=]<latitude>\n"
-                   "              [ ... [--lon9=]<longitude> [--lon9=]<latitude> ]\n"
+                   "              [ ... [--lon99=]<longitude> [--lon99=]<latitude>]\n"
                    "              [--help | --help-profile | --help-profile-js | --help-profile-pl]\n"
                    "              [--dir=<name>] [--prefix=<name>]\n"
                    "              [--shortest | --quickest]\n"
-                   "              [--all | --super]\n"
-                   "              [--no-output] [--quiet]\n"
+                   "              [--quiet]\n"
                    "              [--transport=<transport>]\n"
                    "              [--highway-<highway>=<preference> ...]\n"
                    "              [--speed-<highway>=<speed> ...]\n"
@@ -119,36 +120,48 @@ int main(int argc,char** argv)
     if(isdigit(argv[arg][0]) ||
        ((argv[arg][0]=='-' || argv[arg][0]=='+') && isdigit(argv[arg][1])))
       {
-       for(node=1;node<sizeof(point_used)/sizeof(point_used[0]);node++)
+       for(node=1;node<=NWAYPOINTS;node++)
           if(point_used[node]!=3)
             {
              if(point_used[node]==0)
                {
-                point_lon[node]=(M_PI/180)*atof(argv[arg]);
+                point_lon[node]=degrees_to_radians(atof(argv[arg]));
                 point_used[node]=1;
                }
              else /* if(point_used[node]==1) */
                {
-                point_lat[node]=(M_PI/180)*atof(argv[arg]);
+                point_lat[node]=degrees_to_radians(atof(argv[arg]));
                 point_used[node]=3;
                }
              break;
             }
       }
-    else if(!strncmp(argv[arg],"--lon",5) && isdigit(argv[arg][5]) && argv[arg][6]=='=')
-      {
-       node=atoi(&argv[arg][5]);
-       if(point_used[node]&1)
-          goto usage;
-       point_lon[node]=(M_PI/180)*atof(&argv[arg][7]);
+     else if(!strncmp(argv[arg],"--lon",5) && isdigit(argv[arg][5]))
+       {
+        char *p=&argv[arg][6];
+        while(isdigit(*p)) p++;
+        if(*p++!='=')
+           goto usage;
+ 
+        node=atoi(&argv[arg][5]);
+        if(node>NWAYPOINTS || point_used[node]&1)
+           goto usage;
+ 
+       point_lon[node]=degrees_to_radians(atof(p));
        point_used[node]+=1;
       }
-    else if(!strncmp(argv[arg],"--lat",5) && isdigit(argv[arg][5]) && argv[arg][6]=='=')
-      {
-       node=atoi(&argv[arg][5]);
-       if(point_used[node]&2)
-          goto usage;
-       point_lat[node]=(M_PI/180)*atof(&argv[arg][7]);
+     else if(!strncmp(argv[arg],"--lat",5) && isdigit(argv[arg][5]))
+       {
+        char *p=&argv[arg][6];
+        while(isdigit(*p)) p++;
+        if(*p++!='=')
+           goto usage;
+ 
+        node=atoi(&argv[arg][5]);
+        if(node>NWAYPOINTS || point_used[node]&2)
+           goto usage;
+ 
+       point_lat[node]=degrees_to_radians(atof(p));
        point_used[node]+=2;
       }
     else if(!strcmp(argv[arg],"--help"))
@@ -167,12 +180,6 @@ int main(int argc,char** argv)
        option_quickest=0;
     else if(!strcmp(argv[arg],"--quickest"))
        option_quickest=1;
-    else if(!strcmp(argv[arg],"--all"))
-       option_all=1;
-    else if(!strcmp(argv[arg],"--super"))
-       option_super=1;
-    else if(!strcmp(argv[arg],"--no-output"))
-       option_no_output=1;
     else if(!strcmp(argv[arg],"--quiet"))
        option_quiet=1;
     else if(!strncmp(argv[arg],"--transport=",12))
@@ -233,7 +240,7 @@ int main(int argc,char** argv)
        goto usage;
    }
 
- for(node=0;node<sizeof(point_used)/sizeof(point_used[0]);node++)
+ for(node=0;node<=NWAYPOINTS;node++)
     if(point_used[node]==1 || point_used[node]==2)
        goto usage;
 
@@ -264,7 +271,7 @@ int main(int argc,char** argv)
 
  if(!OSMNodes)
    {
-    fprintf(stderr,"Cannot open nodes file '%s'.\n",filename);
+    fprintf(stderr,"Error: Cannot open nodes file '%s'.\n",filename);
     return(1);
    }
 
@@ -272,7 +279,7 @@ int main(int argc,char** argv)
 
  if(!OSMSegments)
    {
-    fprintf(stderr,"Cannot open segments file '%s'.\n",filename);
+    fprintf(stderr,"Error: Cannot open segments file '%s'.\n",filename);
     return(1);
    }
 
@@ -280,14 +287,15 @@ int main(int argc,char** argv)
 
  if(!OSMWays)
    {
-    fprintf(stderr,"Cannot open ways file '%s'.\n",filename);
+    fprintf(stderr,"Error: Cannot open ways file '%s'.\n",filename);
     return(1);
    }
 
  /* Loop through all pairs of nodes */
 
- for(node=1;node<sizeof(point_used)/sizeof(point_used[0]);node++)
+ for(node=1;node<=NWAYPOINTS;node++)
    {
+    Results *begin,*end;
     distance_t dist=km_to_distance(10);
 
     if(point_used[node]!=3)
@@ -301,136 +309,105 @@ int main(int argc,char** argv)
 
     if(finish==NO_NODE)
       {
-       fprintf(stderr,"Cannot find node close to specified point %d.\n",node);
+       fprintf(stderr,"Error: Cannot find node close to specified point %d.\n",node);
        return(1);
       }
 
     if(!option_quiet)
       {
-       float lat,lon;
+       double lat,lon;
 
        GetLatLong(OSMNodes,finish,&lat,&lon);
 
-       printf("Node %d: %3.6f %4.6f = %2.3f km\n",node,(180.0/M_PI)*lon,(180.0/M_PI)*lat,distance_to_km(dist));
+       printf("Node %d: %3.6f %4.6f = %2.3f km\n",node,radians_to_degrees(lon),radians_to_degrees(lat),distance_to_km(dist));
       }
 
     if(start==NO_NODE)
        continue;
 
-    /* Find the route segment */
+    /* Calculate the beginning of the route */
 
-    if(option_super && !IsSuperNode(OSMNodes,start) && !IsSuperNode(OSMNodes,finish))
+    if(IsSuperNode(OSMNodes,start))
       {
-       fprintf(stderr,"Start and/or finish nodes are not super-nodes.\n");
-       return(1);
-      }
+       Result *result;
 
-    /* Calculate the route. */
+       begin=NewResultsList(1);
 
-    if(option_all)
-      {
-       /* Calculate the route */
+       begin->start=start;
 
-       results[node]=FindRoute(OSMNodes,OSMSegments,OSMWays,start,finish,&profile,option_all);
+       result=InsertResult(begin,start);
 
-       if(!results[node])
-         {
-          fprintf(stderr,"Cannot find route compatible with profile.\n");
-          return(1);
-         }
+       ZeroResult(result);
       }
     else
       {
-       Results *begin,*end;
+       begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,start,&profile);
 
-       /* Calculate the beginning of the route */
+       if(!begin)
+         {
+          fprintf(stderr,"Error: Cannot find initial section of route compatible with profile.\n");
+          return(1);
+         }
+      }
 
-       if(IsSuperNode(OSMNodes,start))
+    if(FindResult(begin,finish))
+      {
+       results[node]=begin;
+
+       results[node]->finish=finish;
+      }
+    else
+      {
+       Results *superresults;
+
+       /* Calculate the end of the route */
+
+       if(IsSuperNode(OSMNodes,finish))
          {
           Result *result;
 
-          begin=NewResultsList(1);
+          end=NewResultsList(1);
 
-          begin->start=start;
+          end->finish=finish;
 
-          result=InsertResult(begin,start);
+          result=InsertResult(end,finish);
 
           ZeroResult(result);
          }
        else
          {
-          begin=FindStartRoutes(OSMNodes,OSMSegments,OSMWays,start,&profile);
+          end=FindFinishRoutes(OSMNodes,OSMSegments,OSMWays,finish,&profile);
 
-          if(!begin)
+          if(!end)
             {
-             fprintf(stderr,"Cannot find initial section of route compatible with profile.\n");
+             fprintf(stderr,"Error: Cannot find final section of route compatible with profile.\n");
              return(1);
             }
          }
 
-       if(FindResult(begin,finish))
+       /* Calculate the middle of the route */
+
+       superresults=FindMiddleRoute(OSMNodes,OSMSegments,OSMWays,begin,end,&profile);
+
+       if(!superresults)
          {
-          results[node]=begin;
-
-          results[node]->finish=finish;
+          fprintf(stderr,"Error: Cannot find route compatible with profile.\n");
+          return(1);
          }
-       else
-         {
-          Results *superresults;
 
-          /* Calculate the end of the route */
-
-          if(IsSuperNode(OSMNodes,finish))
-            {
-             Result *result;
-
-             end=NewResultsList(1);
-
-             end->finish=finish;
-
-             result=InsertResult(end,finish);
-
-             ZeroResult(result);
-            }
-          else
-            {
-             end=FindFinishRoutes(OSMNodes,OSMSegments,OSMWays,finish,&profile);
-
-             if(!end)
-               {
-                fprintf(stderr,"Cannot find final section of route compatible with profile.\n");
-                return(1);
-               }
-            }
-
-          /* Calculate the middle of the route */
-
-          superresults=FindRoute3(OSMNodes,OSMSegments,OSMWays,begin,end,&profile);
-
-          if(!superresults)
-            {
-             fprintf(stderr,"Cannot find route compatible with profile.\n");
-             return(1);
-            }
-
-          if(option_super)
-             results[node]=superresults;
-          else
-             results[node]=CombineRoutes(superresults,OSMNodes,OSMSegments,OSMWays,&profile);
-         }
+       results[node]=CombineRoutes(superresults,OSMNodes,OSMSegments,OSMWays,&profile);
       }
    }
 
- if(!option_no_output)
-   {
-    PrintRouteHead(FileName(dirname,prefix,"copyright.txt"));
+ /* Print out the combined route */
 
-    for(node=1;node<sizeof(point_used)/sizeof(point_used[0]);node++)
-       if(results[node])
-          PrintRoute(results[node],OSMNodes,OSMSegments,OSMWays,&profile);
+ PrintRouteHead(FileName(dirname,prefix,"copyright.txt"));
 
-    PrintRouteTail();
-   }
+ for(node=1;node<=NWAYPOINTS;node++)
+    if(results[node])
+       PrintRoute(results[node],OSMNodes,OSMSegments,OSMWays,&profile);
+
+ PrintRouteTail();
 
  return(0);
 }
