@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/segments.h,v 1.30 2009/04/08 16:54:34 amb Exp $
+ $Header: /home/amb/routino/src/RCS/segments.h,v 1.33 2009/07/19 14:10:27 amb Exp $
 
  A header file for the segments.
 
@@ -51,7 +51,9 @@ struct _Segment
 /*+ A structure containing a set of segments (mmap format). +*/
 struct _Segments
 {
- uint32_t  number;              /*+ How many entries are used in total? +*/
+ uint32_t  number;              /*+ How many segments in total? +*/
+ uint32_t  snumber;             /*+ How many super-segments? +*/
+ uint32_t  nnumber;             /*+ How many normal segments? +*/
 
  Segment  *segments;            /*+ An array of segments. +*/
 
@@ -66,19 +68,19 @@ struct _Segments
 #define LookupSegment(xxx,yyy) (&(xxx)->segments[yyy])
 
 /*+ Return true if this is a normal segment. +*/
-#define IsNormalSegment(xxx)   (((xxx)->node1)&SUPER_FLAG)
+#define IsNormalSegment(xxx)   (((xxx)->distance)&SEGMENT_NORMAL)
 
 /*+ Return true if this is a super-segment. +*/
-#define IsSuperSegment(xxx)    (((xxx)->node2)&SUPER_FLAG)
+#define IsSuperSegment(xxx)    (((xxx)->distance)&SEGMENT_SUPER)
 
 /*+ Return true if the segment is oneway towards the specified node. +*/
-#define IsOnewayTo(xxx,yyy)    ((NODE((xxx)->node1)==(yyy))?((xxx)->distance&ONEWAY_2TO1):((xxx)->distance&ONEWAY_1TO2))
+#define IsOnewayTo(xxx,yyy)    ((xxx)->node1==(yyy)?((xxx)->distance&ONEWAY_2TO1):((xxx)->distance&ONEWAY_1TO2))
 
 /*+ Return true if the segment is oneway from the specified node. +*/
-#define IsOnewayFrom(xxx,yyy)  ((NODE((xxx)->node2)==(yyy))?((xxx)->distance&ONEWAY_2TO1):((xxx)->distance&ONEWAY_1TO2))
+#define IsOnewayFrom(xxx,yyy)  ((xxx)->node2==(yyy)?((xxx)->distance&ONEWAY_2TO1):((xxx)->distance&ONEWAY_1TO2))
 
 /*+ Return the other node in the segment that is not the specified node. +*/
-#define OtherNode(xxx,yyy)     ((NODE((xxx)->node1)==(yyy))?NODE((xxx)->node2):NODE((xxx)->node1))
+#define OtherNode(xxx,yyy)     ((xxx)->node1==(yyy)?(xxx)->node2:(xxx)->node1)
 
 
 /* Functions */
@@ -88,7 +90,7 @@ Segments *LoadSegmentList(const char *filename);
 
 Segment *NextSegment(Segments* segments,Segment *segment,index_t node);
 
-distance_t Distance(float lat1,float lon1,float lat2,float lon2);
+distance_t Distance(double lat1,double lon1,double lat2,double lon2);
 
 duration_t Duration(Segment *segment,Way *way,Profile *profile);
 
