@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/segmentsx.h,v 1.4 2009/06/13 13:02:12 amb Exp $
+ $Header: /home/amb/routino/src/RCS/segmentsx.h,v 1.19 2009/10/08 19:20:29 amb Exp $
 
  A header file for the extended segments.
 
@@ -27,8 +27,8 @@
 
 #include <stdint.h>
 
+#include "typesx.h"
 #include "types.h"
-#include "segments.h"
 
 
 /* Data structures */
@@ -37,32 +37,35 @@
 /*+ An extended structure used for processing. +*/
 struct _SegmentX
 {
- node_t    node1;               /*+ The starting node. +*/
- node_t    node2;               /*+ The finishing node. +*/
+ node_t     node1;              /*+ The id of the starting node. +*/
+ node_t     node2;              /*+ The id of the finishing node. +*/
 
- Segment   segment;             /*+ The real segment data. +*/
+ way_t      way;                /*+ The id of the way. +*/
+
+ distance_t distance;           /*+ The distance between the nodes. +*/
 };
 
 
 /*+ A structure containing a set of segments (memory format). +*/
 struct _SegmentsX
 {
- uint32_t   sorted;             /*+ Is the data sorted and therefore searchable? +*/
- uint32_t   alloced;            /*+ How many entries are allocated? +*/
- uint32_t   xnumber;            /*+ How many entries are used from those allocated? +*/
+ char      *filename;           /*+ The name of the temporary file. +*/
+ int        fd;                 /*+ The file descriptor of the temporary file. +*/
+
+ uint32_t   xnumber;            /*+ The number of unsorted extended nodes. +*/
+
+ SegmentX  *xdata;              /*+ The extended segment data (unsorted). +*/
+ SegmentX   cached[2];          /*+ Two cached segments read from the file in slim mode. +*/
+
  uint32_t   number;             /*+ How many entries are still useful? +*/
 
- SegmentX **sdata;              /*+ The extended segment data (sorted by node). +*/
- SegmentX  *xdata;              /*+ The extended segment data (unsorted). +*/
+ node_t   *idata;               /*+ The extended segment data (sorted by node1 then node2). +*/
+ index_t  *firstnode;           /*+ The first segment index for each node. +*/
+
+ Segment   *sdata;              /*+ The segment data (same order as n1data). +*/
 };
 
 
-/* Macros */
-
-
-#define LookupSegmentX(xxx,yyy) (&(xxx)->sdata[yyy])
-
-
 /* Functions */
 
 
@@ -71,24 +74,27 @@ void FreeSegmentList(SegmentsX *segmentsx);
 
 void SaveSegmentList(SegmentsX *segmentsx,const char *filename);
 
-SegmentX **FindFirstSegmentX(SegmentsX* segmentsx,node_t node);
-SegmentX **FindNextSegmentX(SegmentsX* segmentsx,SegmentX **segmentx);
+SegmentX *LookupSegmentX(SegmentsX* segmentsx,index_t index,int position);
 
-Segment *AppendSegment(SegmentsX* segmentsx,node_t node1,node_t node2);
+index_t IndexFirstSegmentX(SegmentsX* segmentsx,node_t node);
 
-void SortSegmentList(SegmentsX *segmentsx);
+index_t IndexNextSegmentX(SegmentsX* segmentsx,index_t segindex,index_t nodeindex);
+
+void AppendSegment(SegmentsX* segmentsx,way_t way,node_t node1,node_t node2,distance_t distance);
+
+void SortSegmentList(SegmentsX* segmentsx);
 
 void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx);
 
-void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx);
+void UpdateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx);
 
-void RotateSegments(SegmentsX* segmentsx,NodesX *nodesx);
+void RotateSegments(SegmentsX* segmentsx);
 
 void DeduplicateSegments(SegmentsX* segmentsx,NodesX *nodesx,WaysX *waysx);
 
-void IndexSegments(SegmentsX* segmentsx,NodesX *nodesx);
+void CreateRealSegments(SegmentsX *segmentsx,WaysX *waysx);
 
-distance_t DistanceX(NodeX *nodex1,NodeX *nodex2);
+void IndexSegments(SegmentsX* segmentsx,NodesX *nodesx);
 
 
 #endif /* SEGMENTSX_H */
