@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/types.h,v 1.21 2009/05/14 18:02:30 amb Exp $
+ $Header: /home/amb/routino/src/RCS/types.h,v 1.28 2009/07/09 17:44:26 amb Exp $
 
  Type definitions
 
@@ -29,46 +29,54 @@
 #include <math.h>
 
 
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
+
 /* Constants and macros for handling them */
 
 
-/*+ The latitude and longitude conversion factor from float (radians) to integer. +*/
+/*+ The latitude and longitude conversion factor from floating point (radians) to integer. +*/
 #define LAT_LONG_SCALE  (1024*65536)
 
 /*+ The latitude and longitude integer range within each bin. +*/
 #define LAT_LONG_BIN    65536
 
-/*+ Convert a latitude or longitude to a bin number (taking care of rounding). +*/
-#define lat_long_to_bin(xxx) ((int32_t)floorf((xxx)*1024))
 
-/*+ Convert a bin number to a latitude or longitude. +*/
-#define bin_to_lat_long(xxx) ((float)(xxx)/1024)
+/*+ A flag to mark a node as a super-node. +*/
+#define NODE_SUPER   ((index_t)0x80000000)
 
-
-/*+ A flag to mark super-nodes and super-segments. +*/
-#define SUPER_FLAG   ((index_t)0x80000000)
-
-/*+ A node index excluding the super-node flag. +*/
-#define NODE(xxx)    (index_t)((xxx)&(~SUPER_FLAG))
+/*+ A segment index excluding the super-node flag. +*/
+#define SEGMENT(xxx) (index_t)((xxx)&(~NODE_SUPER))
 
 /*+ An undefined node index. +*/
 #define NO_NODE      (~(index_t)0)
 
-/*+ A segment index excluding the super-segment flag. +*/
-#define SEGMENT(xxx) (index_t)((xxx)&(~SUPER_FLAG))
-
 /*+ An undefined segment index. +*/
 #define NO_SEGMENT   (~(index_t)0)
 
+/*+ An undefined way index. +*/
+#define NO_WAY       (~(index_t)0)
 
-/*+ A flag to mark a distance as only applying from node1 to node2. +*/
-#define ONEWAY_1TO2  ((distance_t)0x80000000)
 
-/*+ A flag to mark a distance as only applying from node2 to node1. +*/
-#define ONEWAY_2TO1  ((distance_t)0x40000000)
+/*+ A flag to mark a segment as one-way from node1 to node2. +*/
+#define ONEWAY_1TO2    ((distance_t)0x80000000)
 
-/*+ The real distance ignoring the ONEWAY_* flags. +*/
-#define DISTANCE(xx) (distance_t)((xx)&(~(ONEWAY_1TO2|ONEWAY_2TO1)))
+/*+ A flag to mark a segment as one-way node2 to node1. +*/
+#define ONEWAY_2TO1    ((distance_t)0x40000000)
+
+/*+ A flag to mark a segment as a super-segment. +*/
+#define SEGMENT_SUPER  ((distance_t)0x20000000)
+
+/*+ A flag to mark a segment as a normal segment. +*/
+#define SEGMENT_NORMAL ((distance_t)0x10000000)
+
+/*+ The real distance ignoring the ONEWAY_* and SEGMENT_* flags. +*/
+#define DISTANCE(xx) ((distance_t)(xx)&(~(ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL)))
+
+/*+ The distance flags selecting only the ONEWAY_* and SEGMENT_* flags. +*/
+#define DISTFLAG(xx) ((distance_t)(xx)&(ONEWAY_1TO2|ONEWAY_2TO1|SEGMENT_SUPER|SEGMENT_NORMAL))
 
 
 /*+ A very large almost infinite score. +*/
@@ -78,18 +86,47 @@
 /* Simple Types */
 
 
-/*+ A node identifier. +*/
-typedef uint32_t node_t;
-
-
 /*+ A node, segment or way index. +*/
 typedef uint32_t index_t;
 
 
+/*+ A node latitude or longitude. +*/
+typedef int32_t  latlong_t;
+
+/*+ A node latitude or longitude bin number. +*/
+typedef int16_t  ll_bin_t;
+
 /*+ A node latitude or longitude offset. +*/
 typedef uint16_t ll_off_t;
 
 
+/*+ Conversion from a latlong (integer latitude or longitude) to a bin number. +*/
+#define latlong_to_bin(xxx) (ll_bin_t)((latlong_t)((xxx)&~(LAT_LONG_BIN-1))/LAT_LONG_BIN)
+
+/*+ Conversion from a bin number to a latlong (integer latitude or longitude). +*/
+#define bin_to_latlong(xxx) ((latlong_t)(xxx)*LAT_LONG_BIN)
+
+/*+ Conversion from a latlong (integer latitude or longitude) to a bin offset. +*/
+#define latlong_to_off(xxx) (ll_off_t)((latlong_t)(xxx)&(LAT_LONG_BIN-1))
+
+/*+ Conversion from a bin offset to a latlong (integer latitude or longitude). +*/
+#define off_to_latlong(xxx) ((latlong_t)(xxx))
+
+
+/*+ Conversion from a latitude or longitude in radians to a latlong (integer latitude or longitude). +*/
+#define radians_to_latlong(xxx) ((latlong_t)floor((xxx)*LAT_LONG_SCALE))
+
+/*+ Conversion from a latlong (integer latitude or longitude) to a latitude or longitude in radians. +*/
+#define latlong_to_radians(xxx) ((double)(xxx)/LAT_LONG_SCALE)
+
+
+/*+ Conversion from radians to degrees. +*/
+#define radians_to_degrees(xxx) ((xxx)*(180.0/M_PI))
+
+/*+ Conversion from degrees to radians. +*/
+#define degrees_to_radians(xxx) ((xxx)*(M_PI/180.0))
+
+
 /*+ A distance, measured in metres. +*/
 typedef uint32_t distance_t;
 
@@ -209,7 +246,7 @@ typedef uint8_t length_t;
 #define kph_to_speed(xxx)      (speed_t)(xxx)
 
 /*+ Conversion of speed_t to km/hr. +*/
-#define speed_to_kph(xxx)      (xxx)
+#define speed_to_kph(xxx)      (int)(xxx)
 
 /*+ Conversion of tonnes to weight_t. +*/
 #define tonnes_to_weight(xxx)  (weight_t)((xxx)*5)
@@ -236,10 +273,6 @@ typedef uint8_t length_t;
 #define length_to_metres(xxx)  ((double)(xxx)/10.0)
 
 
-/*+ The preference scaling for a highway type. +*/
-typedef uint8_t invpref_t;
-
-
 /* Data structures */
 
 typedef struct _Node Node;
@@ -254,17 +287,5 @@ typedef struct _Way Way;
 
 typedef struct _Ways Ways;
 
-typedef struct _NodeX NodeX;
-
-typedef struct _NodesX NodesX;
-
-typedef struct _SegmentX SegmentX;
-
-typedef struct _SegmentsX SegmentsX;
-
-typedef struct _WayX WayX;
-
-typedef struct _WaysX WaysX;
-
 
 #endif /* TYPES_H */
