@@ -1,5 +1,5 @@
 /***************************************
- $Header: /home/amb/routino/src/RCS/ways.h,v 1.27 2009/05/31 12:30:12 amb Exp $
+ $Header: /home/amb/routino/src/RCS/ways.h,v 1.31 2009/09/17 12:53:58 amb Exp $
 
  A header file for the ways.
 
@@ -48,15 +48,18 @@ struct _Way
  height_t   height;             /*+ The defined maximum height of traffic on the way. +*/
  width_t    width;              /*+ The defined maximum width of traffic on the way. +*/
  length_t   length;             /*+ The defined maximum length of traffic on the way. +*/
+
+ uint8_t    padding;            /*+ Padding to make a multiple of 4 bytes +*/
 };
 
 
 /*+ A structure containing a set of ways (mmap format). +*/
 struct _Ways
 {
- uint32_t number;               /*+ How many entries are used? +*/
+ uint32_t number;               /*+ How many ways are stored? +*/
+ uint32_t onumber;              /*+ How many ways were there originally? +*/
 
- Way     *ways;                 /*+ An array of ways. */
+ Way     *ways;                 /*+ An array of ways. +*/
  char    *names;                /*+ An array of characters containing the names. +*/
 
  void    *data;                 /*+ The memory mapped data. +*/
@@ -87,7 +90,7 @@ const char *TransportName(Transport transport);
 const char *HighwayList(void);
 const char *TransportList(void);
 
-int WaysSame(Way *way1,Way *way2);
+int WaysCompare(Way *way1,Way *way2);
 
 
 #endif /* WAYS_H */
